@@ -33,6 +33,8 @@
 (require 'riece-highlight)
 (require 'riece-display)
 (require 'riece-debug)
+(require 'riece-mcat)
+(require 'riece-message)
 
 (defface riece-ctcp-action-face
   '((((class color)
@@ -104,7 +106,7 @@
        riece-dialogue-buffer)
      (concat
       (riece-concat-server-name
-       (format "CTCP VERSION from %s (%s) to %s"
+       (format (riece-mcat "CTCP VERSION from %s (%s) to %s")
 	       user
 	       (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	       (riece-format-identity target-identity t)))
@@ -118,7 +120,7 @@
     (riece-send-string
      (if string
 	 (format "NOTICE %s :\1PING %s\1\r\n" user string)
-       (format "NOTICE %s :\1PING\1\r\n" user string)))
+       (format "NOTICE %s :\1PING\1\r\n" user)))
     (riece-insert-change buffer (format "CTCP PING from %s\n" user))
     (riece-insert-change
      (if (and riece-channel-buffer-mode
@@ -127,7 +129,7 @@
        riece-dialogue-buffer)
      (concat
       (riece-concat-server-name
-       (format "CTCP PING from %s (%s) to %s"
+       (format (riece-mcat "CTCP PING from %s (%s) to %s")
 	       user
 	       (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	       (riece-format-identity target-identity t)))
@@ -163,38 +165,41 @@
        riece-dialogue-buffer)
      (concat
       (riece-concat-server-name
-       (format "CTCP CLIENTINFO from %s (%s) to %s"
+       (format (riece-mcat "CTCP CLIENTINFO from %s (%s) to %s")
 	       user
 	       (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	       (riece-format-identity target-identity t)))
       "\n"))))
 
+(defun riece-ctcp-action-format-message (message &optional global)
+  (riece-with-server-buffer (riece-identity-server
+			     (riece-message-speaker message))
+    (concat
+     (if global
+	 (riece-concat-server-name
+	  (concat riece-ctcp-action-prefix
+		  (riece-format-identity (riece-message-target message) t) ": "
+		  (riece-identity-prefix (riece-message-speaker message)) " "
+		  (riece-message-text message)))
+       (concat riece-ctcp-action-prefix
+	       (riece-identity-prefix (riece-message-speaker message)) " "
+	       (riece-message-text message)))
+     "\n")))
+
 (defun riece-handle-ctcp-action-request (prefix target string)
   (let ((buffer (if (riece-channel-p target)
 		    (riece-channel-buffer (riece-make-identity
 					   target riece-server-name))))
 	(user (riece-prefix-nickname prefix)))
-    (riece-insert buffer (concat riece-ctcp-action-prefix
-				 (riece-format-identity
-				  (riece-make-identity user riece-server-name)
-				  t)
-				 " " string
-				 "\n"))
-    (riece-insert
-     (if (and riece-channel-buffer-mode
-	      (not (eq buffer riece-channel-buffer)))
-	 (list riece-dialogue-buffer riece-others-buffer)
-       riece-dialogue-buffer)
-     (concat (riece-concat-server-name
-	      (concat riece-ctcp-action-prefix
-		      (riece-format-identity
-		       (riece-make-identity target riece-server-name)
-		       t)
-		      ": "
-		      (riece-format-identity
-		       (riece-make-identity user riece-server-name)
-		       t)
-		      " " string)) "\n"))))
+    (riece-display-message
+     (riece-make-message (riece-make-identity user
+					      riece-server-name)
+			 (riece-make-identity target
+					      riece-server-name)
+			 string
+			 'action
+			 (riece-identity-equal-no-server
+			  user riece-real-nickname)))))
 
 (defun riece-handle-ctcp-time-request (prefix target string)
   (let* ((target-identity (riece-make-identity target riece-server-name))
@@ -204,7 +209,8 @@
 	 (time (format-time-string "%c")))
     (riece-send-string
      (format "NOTICE %s :\1TIME %s\1\r\n" user time))
-    (riece-insert-change buffer (format "CTCP TIME from %s\n" user))
+    (riece-insert-change buffer (format (riece-mcat "CTCP TIME from %s\n")
+					user))
     (riece-insert-change
      (if (and riece-channel-buffer-mode
 	      (not (eq buffer riece-channel-buffer)))
@@ -212,7 +218,7 @@
        riece-dialogue-buffer)
      (concat
       (riece-concat-server-name
-       (format "CTCP TIME from %s (%s) to %s"
+       (format (riece-mcat "CTCP TIME from %s (%s) to %s")
 	       user
 	       (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	       (riece-format-identity target-identity t)))
@@ -254,7 +260,7 @@
    (list riece-dialogue-buffer riece-others-buffer)
    (concat
     (riece-concat-server-name
-     (format "CTCP VERSION for %s (%s) = %s"
+     (format (riece-mcat "CTCP VERSION for %s (%s) = %s")
 	     (riece-prefix-nickname prefix)
 	     (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	     string))
@@ -268,7 +274,7 @@
      (list riece-dialogue-buffer riece-others-buffer)
      (concat
       (riece-concat-server-name
-       (format "CTCP PING for %s (%s) = %d sec"
+       (format (riece-mcat "CTCP PING for %s (%s) = %d sec")
 	       (riece-prefix-nickname prefix)
 	       (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	       elapsed))
@@ -279,7 +285,7 @@
    (list riece-dialogue-buffer riece-others-buffer)
    (concat
     (riece-concat-server-name
-     (format "CTCP CLIENTINFO for %s (%s) = %s"
+     (format (riece-mcat "CTCP CLIENTINFO for %s (%s) = %s")
 	     (riece-prefix-nickname prefix)
 	     (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	     string))
@@ -290,7 +296,7 @@
    (list riece-dialogue-buffer riece-others-buffer)
    (concat
     (riece-concat-server-name
-     (format "CTCP TIME for %s (%s) = %s"
+     (format (riece-mcat "CTCP TIME for %s (%s) = %s")
 	     (riece-prefix-nickname prefix)
 	     (riece-strip-user-at-host (riece-prefix-user-at-host prefix))
 	     string))
@@ -299,7 +305,7 @@
 (defun riece-command-ctcp-version (target)
   (interactive
    (list (riece-completing-read-identity
-	  "Channel/User: "
+	  (riece-mcat "Channel/User: ")
 	  (riece-get-identities-on-server (riece-current-server-name)))))
   (riece-send-string (format "PRIVMSG %s :\1VERSION\1\r\n"
 			     (riece-identity-prefix target))))
@@ -307,7 +313,7 @@
 (defun riece-command-ctcp-ping (target)
   (interactive
    (list (riece-completing-read-identity
-	  "Channel/User: "
+	  (riece-mcat "Channel/User: ")
 	  (riece-get-identities-on-server (riece-current-server-name)))))
   (riece-send-string (format "PRIVMSG %s :\1PING\1\r\n"
 			     (riece-identity-prefix target)))
@@ -316,7 +322,7 @@
 (defun riece-command-ctcp-clientinfo (target)
   (interactive
    (list (riece-completing-read-identity
-	  "Channel/User: "
+	  (riece-mcat "Channel/User: ")
 	  (riece-get-identities-on-server (riece-current-server-name)))))
   (riece-send-string (format "PRIVMSG %s :\1CLIENTINFO\1\r\n"
 			     (riece-identity-prefix target))))
@@ -325,7 +331,7 @@
   (interactive
    (list (if current-prefix-arg
 	     (riece-completing-read-identity
-	      "Channel/User: "
+	      (riece-mcat "Channel/User: ")
 	      (riece-get-identities-on-server (riece-current-server-name)))
 	   riece-current-channel)
 	 (let (message)
@@ -333,8 +339,9 @@
 	   (setq message (buffer-substring (point)
 					   (progn (end-of-line) (point))))
 	   (if (equal message "")
-	       (read-string "Action: ")
-	     (prog1 (read-from-minibuffer "Action: " (cons message 0))
+	       (read-string (riece-mcat "Action: "))
+	     (prog1 (read-from-minibuffer (riece-mcat "Action: ")
+					  (cons message 0))
 	       (let ((next-line-add-newlines t))
 		 (next-line 1)))))))
   (if (equal action "")
@@ -342,28 +349,13 @@
   (riece-send-string (format "PRIVMSG %s :\1ACTION %s\1\r\n"
 			     (riece-identity-prefix target)
 			     action))
-  (let ((buffer (riece-channel-buffer target)))
-    (riece-insert
-     buffer
-     (concat riece-ctcp-action-prefix
-	     (riece-identity-prefix (riece-current-nickname)) " " action "\n"))
-    (riece-insert
-     (if (and riece-channel-buffer-mode
-	      (not (eq buffer riece-channel-buffer)))
-	 (list riece-dialogue-buffer riece-others-buffer)
-       riece-dialogue-buffer)
-     (concat
-      (riece-with-server-buffer (riece-identity-server target)
-	(riece-concat-server-name
-	 (concat riece-ctcp-action-prefix
-		 (riece-format-identity target t) ": "
-		 (riece-identity-prefix (riece-current-nickname)) " " action)))
-      "\n"))))
+  (riece-display-message
+   (riece-make-message (riece-current-nickname) target action 'action t)))
 
 (defun riece-command-ctcp-time (target)
   (interactive
    (list (riece-completing-read-identity
-	  "Channel/User: "
+	  (riece-mcat "Channel/User: ")
 	  (riece-get-identities-on-server (riece-current-server-name)))))
   (riece-send-string (format "PRIVMSG %s :\1TIME\1\r\n"
 			     (riece-identity-prefix target))))
@@ -384,7 +376,11 @@
   (if (memq 'riece-highlight riece-addons)
       (setq riece-dialogue-font-lock-keywords
 	    (cons riece-ctcp-dialogue-font-lock-keywords
-		  riece-dialogue-font-lock-keywords))))
+		  riece-dialogue-font-lock-keywords)))
+  (unless (assq 'action riece-message-format-function-alist)
+    (setq riece-message-format-function-alist
+	  (cons (cons 'action #'riece-ctcp-action-format-message)
+		riece-message-format-function-alist))))
 
 (defun riece-ctcp-uninstall ()
   (remove-hook 'riece-privmsg-hook 'riece-handle-ctcp-request)
